"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const moment_1 = __importDefault(require("moment"));
const testrail_js_api_1 = require("testrail-js-api");
const config_1 = require("./config");
const utils_1 = require("./utils");
const Status = {
    Passed: {
        value: testrail_js_api_1.TestStatus.Passed,
        text: "PASSED",
        color: "yellow",
    },
    Blocked: {
        value: testrail_js_api_1.TestStatus.Blocked,
        text: "SKIPPED",
        color: "green",
    },
    Failed: {
        value: testrail_js_api_1.TestStatus.Failed,
        text: "FAILED",
        color: "red",
    },
};
const prepareRun = async (testrailAPI, config, runName, refs, caseIdList) => {
    const { projectId, suiteId, runDescription, runId, updateRunTestCases } = config;
    let existingRun;
    if (runId) {
        const { value: returnedRun } = await utils_1.throwOnApiError(testrailAPI.getRun(runId));
        existingRun = returnedRun;
    }
    else {
        const { value: runs } = await utils_1.throwOnApiError(testrailAPI.getRuns(projectId, { is_completed: 0 }));
        existingRun = runs === null || runs === void 0 ? void 0 : runs.find((run) => run.refs === refs);
    }
    if (!updateRunTestCases) {
        if (existingRun) {
            return existingRun;
        }
        else {
            throw new Error(`[TestRail] Flag 'updateRunTestCases' enabled but the run was not found, please create it`);
        }
    }
    else if (existingRun) {
        const { value: tests } = await utils_1.throwOnApiError(testrailAPI.getTests(existingRun.id));
        const currentCaseIds = (tests === null || tests === void 0 ? void 0 : tests.map((test) => test.case_id)) || [];
        const additionalDescription = "\n" + runDescription;
        const newDescription = existingRun.description
            ? existingRun.description.replace(additionalDescription, "") +
                additionalDescription
            : additionalDescription;
        await utils_1.throwOnApiError(testrailAPI.updateRun(existingRun.id, {
            description: newDescription,
            case_ids: [...currentCaseIds, ...caseIdList],
        }));
        console.log(`[TestRail] Test run updated successfully: ${runName}`);
        return existingRun;
    }
    else {
        const payload = {
            suite_id: suiteId,
            include_all: false,
            case_ids: caseIdList,
            name: runName,
            description: runDescription,
            refs,
        };
        const { value: newRun } = await utils_1.throwOnApiError(testrailAPI.addRun(projectId, payload));
        console.log(`[TestRail] Test run added successfully: ${runName}`);
        return newRun;
    }
};
const closeOldRuns = async (testrailAPI, config) => {
    if (config.runCloseAfterDays) {
        const { value: runs } = await utils_1.throwOnApiError(testrailAPI.getRuns(config.projectId, { is_completed: 0 }));
        if (runs.length) {
            for (let i = 0; i < runs.length; i++) {
                const shouldClose = moment_1.default.unix(runs[i].created_on) <=
                    moment_1.default().subtract(config.runCloseAfterDays, "days");
                if (shouldClose) {
                    console.log(`[TestRail] Closing test run ${runs[i].id}: ${runs[i].name}`);
                    await utils_1.throwOnApiError(testrailAPI.closeRun(runs[i].id));
                }
            }
        }
    }
};
class TestcafeTestrailReporter {
    constructor() {
        this.reportTaskStart = async (_startTime, userAgents) => {
            this.userAgents = userAgents;
        };
        this.reportFixtureStart = async () => {
        };
        this.reportTestDone = async (name, testRunInfo, meta, formatError) => {
            var _a;
            const hasErr = testRunInfo.errs.length;
            let testStatus = null;
            if (testRunInfo.skipped) {
                testStatus = Status.Blocked;
            }
            else if (hasErr === 0) {
                testStatus = Status.Passed;
            }
            else {
                testStatus = Status.Failed;
            }
            let caseId = 0;
            if (meta[this.config.caseMeta]) {
                caseId = Number(meta[this.config.caseMeta].replace("C", "").trim());
            }
            if (caseId > 0) {
                const errorLog = testRunInfo.errs
                    .map((x) => {
                    const formatted = formatError(x).replace(/[\u001b\u009b][[()#;?]*(?:[0-9]{1,4}(?:;[0-9]{0,4})*)?[0-9A-ORZcf-nqry=><]/g, "");
                    return formatted;
                })
                    .join("\n");
                this.results.push({
                    case_id: caseId,
                    status_id: testStatus.value,
                    comment: `Test ${testStatus.text}\n${errorLog}`,
                });
                if (testRunInfo.screenshots.length) {
                    this.screenshots[caseId] = testRunInfo.screenshots;
                }
                if ((_a = testRunInfo.videos) === null || _a === void 0 ? void 0 : _a.length) {
                    this.videos[caseId] = testRunInfo.videos;
                }
            }
            else {
                console.log(`[TestRail] Test missing the TestRail Case ID in test metadata: ${name}`);
            }
        };
        this.reportTaskDone = async (_endTime, _passed, _warnings, _result) => {
            const { host, user, apiKey, projectId, suiteId } = this.config;
            if (config_1.verifyConfig(this.config)) {
                try {
                    if (this.results.length) {
                        const runName = utils_1.prepareReportName(this.config, this.branch, this.buildNo, this.userAgents);
                        const refs = utils_1.prepareReference(this.config, this.branch, this.buildNo);
                        const caseIdList = this.results.map((result) => result.case_id);
                        const testrailAPI = new testrail_js_api_1.TestRail(host, user, apiKey);
                        const { value: caseList } = await utils_1.throwOnApiError(testrailAPI.getCases(projectId, { suite_id: suiteId }));
                        const existingCaseIds = caseList.map((item) => item.id);
                        caseIdList.forEach((id) => {
                            if (!existingCaseIds.includes(id)) {
                                console.error(`[TestRail] All TestRail mappings should be valid. Following test case id does not exist in TestRail: ${id}.`);
                            }
                        });
                        const run = await prepareRun(testrailAPI, this.config, runName, refs, caseIdList);
                        await this.publishTestResults(testrailAPI, run, this.results);
                        await closeOldRuns(testrailAPI, this.config);
                    }
                    else {
                        console.log("[TestRail] No test case data found to publish");
                    }
                }
                catch (error) {
                    console.error("[TestRail] Sending report to TestRail failed", error);
                    throw error;
                }
            }
        };
        this.publishTestResults = async (testrailAPI, run, resultsToPush) => {
            const runId = run.id;
            const { value: results } = await utils_1.throwOnApiError(testrailAPI.addResultsForCases(runId, resultsToPush));
            const { value: tests } = await utils_1.throwOnApiError(testrailAPI.getTests(runId));
            utils_1.uploadScreenshots({
                config: this.config,
                tests,
                results,
                resultsToPush,
                screenshots: this.screenshots,
                testrailAPI,
            });
            utils_1.uploadVideos({
                config: this.config,
                tests,
                results,
                resultsToPush,
                videos: this.videos,
                testrailAPI,
            });
            if (results.length == 0) {
                console.log("[TestRail] No data has been published to TestRail");
            }
            else {
                console.log("[TestRail] Test results added to the TestRail successfully");
            }
        };
        this.config = config_1.prepareConfig();
        this.branch = process.env[this.config.branchEnv] || "master";
        this.buildNo = process.env[this.config.buildNoEnv] || "unknown";
        this.noColors = false;
        this.results = [];
        this.screenshots = {};
        this.videos = {};
    }
}
module.exports = () => {
    const reporter = new TestcafeTestrailReporter();
    return {
        reportTaskStart: reporter.reportTaskStart,
        reportFixtureStart: reporter.reportFixtureStart,
        async reportTestDone(name, testRunInfo, meta) {
            return reporter.reportTestDone(name, testRunInfo, meta, this.formatError.bind(this));
        },
        reportTaskDone: reporter.reportTaskDone,
        reporter,
    };
};
